#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "misterproper.h"

GtkWidget *calendar2;
GtkWidget *clist_taches;
GtkWidget *dial1;
GtkWidget *app1;
GtkWidget *combo_entry1, *hscale1, *dateedit1;
gint selrow = -1;

gboolean
on_misterproper_delete_event(GtkWidget * widget,
							 GdkEvent * event, gpointer user_data)
{
	gtk_main_quit();
	return FALSE;
}


gboolean
on_misterproper_destroy_event(GtkWidget * widget,
							  GdkEvent * event, gpointer user_data)
{
	gtk_main_quit();
	return FALSE;
}


void on_new_clicked(GtkButton * button, gpointer user_data)
{
	gtk_widget_show(taskedit);
}


void on_open_clicked(GtkButton * button, gpointer user_data)
{

}


void on_save_clicked(GtkButton * button, gpointer user_data)
{

}


void on_delete_clicked(GtkButton * button, gpointer user_data)
{
	if (selrow >= 0) {
		Thing *zing;
		zing = gtk_clist_get_row_data(GTK_CLIST(clist_taches), selrow);
		gtk_clist_remove(GTK_CLIST(clist_taches), selrow);
		things = g_slist_remove(things, zing);
		g_free(zing);
		rowz--;
		gtk_dial_set_value(GTK_DIAL(dial1), (gfloat) rowz);
		gtk_calendar_clear_marks(GTK_CALENDAR(calendar2));

		selrow = -1;
	}
}


static void update_delay(const Thing * zing)
{
	gchar z[LINE_MAX];

	g_snprintf(z, sizeof z, _("%llu days"),
			   DAYSINSEC(((unsigned long long) zing->frequency)));
	gtk_clist_set_text(GTK_CLIST(clist_taches), selrow, 1, z);
	update_calendar_marks(selrow, calendar2);
}

void on_less_clicked(GtkButton * button, gpointer user_data)
{
	if (selrow >= 0) {
		Thing *zing =
			gtk_clist_get_row_data(GTK_CLIST(clist_taches), selrow);
		zing->frequency += INCREMENT;
		update_delay(zing);
	}
}


void on_more_clicked(GtkButton * button, gpointer user_data)
{
	if (selrow >= 0) {
		Thing *zing =
			gtk_clist_get_row_data(GTK_CLIST(clist_taches), selrow);
		if (zing->frequency > INCREMENT) {
			zing->frequency -= INCREMENT;
		} else {
			zing->frequency = INCREMENT;
		}
		update_delay(zing);
	}
}


void on_done_clicked(GtkButton * button, gpointer user_data)
{
	if (selrow >= 0) {
		Thing *zing;
		zing = gtk_clist_get_row_data(GTK_CLIST(clist_taches), selrow);
		zing->last = time(NULL);
		gtk_clist_remove(GTK_CLIST(clist_taches), selrow);
		rowz--;
		gtk_dial_set_value(GTK_DIAL(dial1), (gfloat) rowz);

		selrow = -1;
	}
}


void on_Quit_clicked(GtkButton * button, gpointer user_data)
{
	gtk_main_quit();
}

void on_calendar2_realize(GtkWidget * widget, gpointer user_data)
{
	calendar2 = widget;
	reset_calendar_date(calendar2);
}

void on_calendar2_change(GtkWidget * widget, gpointer user_data)
{
	GDate *now;
	guint year, month, day;

	now = g_date_new();
	g_date_set_time(now, time(NULL));
	gtk_calendar_get_date(GTK_CALENDAR(widget), &year, &month, &day);
	month++;
	if (year == g_date_year(now) && month == g_date_month(now)) {
		update_calendar_marks(selrow, widget);
	} else {
		gtk_calendar_clear_marks(GTK_CALENDAR(widget));
	}
	g_date_free(now);
}

void on_clist_taches_realize(GtkWidget * widget, gpointer user_data)
{
	clist_taches = widget;
}


void
on_clist_taches_select_row(GtkCList * clist,
						   gint row,
						   gint column,
						   GdkEvent * event, gpointer user_data)
{
	selrow = row;
	g_assert(calendar2 != NULL);
	update_calendar_marks(selrow, calendar2);
}


void on_dial1_realize(GtkWidget * widget, gpointer user_data)
{
	dial1 = widget;
}


void on_appbar1_realize(GtkWidget * widget, gpointer user_data)
{
	app1 = widget;
	gnome_appbar_set_default(GNOME_APPBAR(widget), _("Okay..."));
}

void on_combo_entry1_realize(GtkWidget * widget, gpointer user_data)
{
	combo_entry1 = widget;
}


void on_hscale1_realize(GtkWidget * widget, gpointer user_data)
{
	hscale1 = widget;
}


void on_dateedit1_realize(GtkWidget * widget, gpointer user_data)
{
	dateedit1 = widget;
}


void on_taskedit_ok_clicked(GtkButton * button, gpointer user_data)
{
	char *name = gtk_entry_get_text(GTK_ENTRY(combo_entry1));
	const time_t freq = (time_t) GTK_RANGE(hscale1)->old_value;
	const time_t next_time =
		gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit1));
	Thing *thing;

	if (name != NULL && *name != 0) {
		thing = g_new(Thing, 1);
		thing->frequency = SECINDAYS(freq);
		g_assert((thing->last = next_time - thing->frequency) > 0);
		stripsep(name);
		thing->name = g_strdup(name);
		things = g_slist_append(things, thing);
		insert_task(thing, 0ULL);
	}
	gtk_widget_hide(taskedit);
}


void on_taskedit_cancel_clicked(GtkButton * button, gpointer user_data)
{
	gtk_widget_hide(taskedit);
}


gint on_taskedit_close(GnomeDialog * gnomedialog, gpointer user_data)
{
	gtk_widget_hide(taskedit);
	return TRUE;
}


gboolean
on_taskedit_destroy_event(GtkWidget * widget,
						  GdkEvent * event, gpointer user_data)
{
	gtk_widget_hide(taskedit);
	return TRUE;
}
