#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <getopt.h>
#include <gnome.h>

#include "interface.h"
#include "support.h"

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <time.h>
#include <locale.h>
#include <sys/stat.h>
#define IN_MAIN
#include "misterproper.h"

extern GtkWidget *clist_taches;
extern GtkWidget *dial1;

static char *bestpath(char *arg)
{
	int points = 0;
	struct stat stz;
	char *pat = NULL;
	const char *home;

	if (arg != NULL) {
		if (access(arg, F_OK) < 0) {
			return arg;
		}
		if (access(arg, R_OK) >= 0) {
			return arg;
		}
	}
	if (stat(FULL_DBPATH, &stz) >= 0) {
		points++;
	}
	if (access(FULL_DBPATH, R_OK) >= 0) {
		points++;
	}
	if (access(FULL_DBPATH, W_OK) >= 0) {
		points++;
	}
	if ((home = g_get_home_dir()) != NULL) {
		pat = g_strconcat(home, "/." DATABASE, NULL);
		if (stat(pat, &stz) >= 0) {
			points--;
		}
		if (access(pat, R_OK) >= 0) {
			points--;
		}
		if (access(pat, W_OK) >= 0) {
			points--;
		}
	}
	if (points > 0) {
		g_free(pat);
		return FULL_DBPATH;
	}
	g_assert(pat != NULL);

	return pat;
}

static int chomp(char *const s)
{
	char *l;

	if (s == NULL || *s == 0) {
		return 0;
	}
	if (*(l = s + strlen(s) - 1) == '\n') {
		*l = 0;
		return 1;
	}
	return 0;
}

void stripsep(char *x)
{
	while (*x != 0) {
		if (*x == *DBSEP) {
			*x = ' ';
		}
		x++;
	}
}

int insert_task(Thing * const current, unsigned long long cnt)
{
	char timebuf[LINE_MAX];
	gchar *tagueule[4];
	const struct tm *tm = localtime(&current->last);

	tagueule[1] = g_malloc(LINE_MAX);
	tagueule[2] = g_malloc(LINE_MAX);
	tagueule[3] = g_malloc(LINE_MAX);
	tagueule[0] = current->name;
	g_snprintf(tagueule[1], LINE_MAX, _("%llu days"),
			   DAYSINSEC((unsigned long long) current->frequency));
	strftime(timebuf, sizeof timebuf, _("%a %B %d %Y , %Hh"), tm);
	g_snprintf(tagueule[2], LINE_MAX, "%s", timebuf);
	if (cnt == 0ULL) {
		strncpy(tagueule[3], _("*New*"), LINE_MAX);
	} else {
		g_snprintf(tagueule[3], LINE_MAX, "%llu",
				   (unsigned long long) cnt);
	}
	gtk_clist_insert(GTK_CLIST(clist_taches), rowz, tagueule);
	gtk_clist_set_row_data(GTK_CLIST(clist_taches), rowz, current);
	rowz++;
	gtk_dial_set_value(GTK_DIAL(dial1), (gfloat) rowz);
	g_free(tagueule[1]);
	g_free(tagueule[2]);
	g_free(tagueule[3]);

	return 0;
}

void reset_calendar_date(GtkWidget * const calendar)
{
	GDate *now;

	now = g_date_new();
	g_date_set_time(now, time(NULL));
	gtk_calendar_clear_marks(GTK_CALENDAR(calendar));
	gtk_calendar_select_month(GTK_CALENDAR(calendar),
							  (guint) g_date_month(now) - 1U,
							  (guint) g_date_year(now));
	gtk_calendar_select_day(GTK_CALENDAR(calendar),
							(guint) g_date_day(now));

	g_date_free(now);
}

void update_calendar_marks(gint row, GtkWidget * const calendar)
{
	Thing *zing;
	GDate *now;
	time_t frq;
	time_t now_time;
	GDateYear year;
	GDateMonth month;
	GDateDay day;

	reset_calendar_date(calendar);
	if (row < 0) {
		return;
	}
	now_time = time(NULL);
	now = g_date_new();
	g_date_set_time(now, now_time);
	year = g_date_year(now);
	month = g_date_month(now);
	day = g_date_day(now);
	zing = gtk_clist_get_row_data(GTK_CLIST(clist_taches), row);
	frq = zing->frequency;
	for (;;) {
		now_time += frq;
		g_date_set_time(now, now_time);
		if (month != g_date_month(now) || year != g_date_year(now)) {
			break;
		}
		if (day != g_date_day(now)) {
			gtk_calendar_mark_day(GTK_CALENDAR(calendar),
								  (guint) g_date_day(now));
		}
	}
	if (zing->last != (time_t) 0U) {
		g_date_set_time(now, zing->last);
		if (g_date_month(now) == month && g_date_year(now) == year) {
			gtk_calendar_mark_day(GTK_CALENDAR(calendar),
								  (guint) g_date_day(now));
		}
	}

	g_date_free(now);
}

int main(int argc, char *argv[])
{
	GtkWidget *misterproper;
	size_t nbent = (size_t) 0U;
	char *database_path;
	char *opt_dbfile = NULL;
	gboolean opt_reset_dates = FALSE;
	const struct poptOption gnopts[] = {
		{
			"dbfile", 'f', POPT_ARG_STRING, &opt_dbfile, 0,
				N_("Sets the database file path"), N_("path")
		},
		{
			"reset", 'r', POPT_ARG_NONE, &opt_reset_dates, 0,
				N_("Reset all dates"), NULL
		},
		{ NULL, 0, 0, NULL, 0, NULL, NULL }
	};
	poptContext ctx;
	
#ifdef ENABLE_NLS
	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain(PACKAGE);
#endif
	srand((unsigned) time(NULL) ^ (unsigned) getpid());
	gnome_init_with_popt_table(PACKAGE, VERSION, argc, argv, gnopts,
							   0, &ctx);	
	database_path = bestpath(opt_dbfile);
	{
		FILE *fp;
		char line[LINE_MAX];

		if ((fp = fopen(database_path, "rt")) == NULL) {
			g_warning(_
					  ("Unable to open the database file, creating an empty list\n"));
		} else {
			while (fgets(line, sizeof line, fp) != NULL) {
				char *skunk;
				Thing *thing;

				thing = g_new(Thing, 1);
				chomp(line);
				if ((skunk = strtok(line, DBSEP)) == NULL) {
					g_warning(_
							  ("Corrupted database : wrong line format\n"));
					g_free(thing);
					continue;
				}
				if ((thing->frequency = (time_t) strtoull(skunk, NULL, 10))
					== (time_t) 0ULL) {
					g_warning(_("Corrupted database : null frequency\n"));
					g_free(thing);
					continue;
				}
				if ((skunk = strtok(NULL, DBSEP)) == NULL) {
					g_warning(_
							  ("Corrupted database : wrong line format\n"));
					g_free(thing);
					continue;
				}
				if (opt_reset_dates != FALSE) {
					thing->last = time(NULL) + 
						(rand() % (thing->frequency * 4)) - 
						(thing->frequency * 2) + thing->frequency / 2;
				} else {
					thing->last = (time_t) strtoull(skunk, NULL, 10);
				}
				if ((skunk = strtok(NULL, DBSEP)) == NULL) {
					g_warning(_
							  ("Corrupted database : wrong line format\n"));
					g_free(thing);
					continue;
				}
				if (*skunk == 0) {
					g_warning(_("Corrupted database : No name\n"));
					g_free(thing);
					continue;
				}
				thing->name = g_strdup(skunk);
				things = g_slist_append(things, thing);
				nbent++;
			}
			fclose(fp);
		}
	}
	misterproper = create_misterproper();
	taskedit = create_taskedit();
	gtk_widget_show(misterproper);

	{
		GSList *skan;
		Thing *current;
		time_t now = time(NULL);
		size_t cnt = (size_t) 1U;

		rowz = 0;
		skan = g_slist_nth(things, 0);
		while (skan != NULL) {
			double elapsed;
			current = skan->data;
			elapsed = difftime(now, current->last + current->frequency);
			if (current->last == (time_t) 0U || elapsed >= 0.0) {
				insert_task(current, cnt);
			}
			cnt++;
			skan = g_slist_next(skan);
		}
	}

	gtk_main();

	{
		Thing *current;
		GSList *skan;
		FILE *fp;

		if ((fp = fopen(database_path, "wt")) == NULL) {
			g_error(_("Unable to save the database\n"));
			return -1;
		}
		skan = g_slist_nth(things, 0);
		while (skan != NULL) {
			current = skan->data;
			g_assert(current != NULL);
			fprintf(fp, "%llu" DBSEP "%llu" DBSEP "%s\n",
					(unsigned long long) current->frequency,
					(unsigned long long) current->last, current->name);
			skan = g_slist_next(skan);
		}
		fclose(fp);
	}

	return 0;
}
